﻿using System;
using System.Collections.Generic;
using Amplitude.AI.Core.AIPlayers.MajorEmpire;
using Amplitude.AI.Core.Brain.Allocators;
using Amplitude.AI.Core.Brain.Goals.Front;
using Amplitude.AI.Core.Brain.Tasks.FleetMissionTasks.FrontBased;
using Amplitude.AI.Core.Decision;
using Amplitude.AI.Core.Decision.ProcessingPasses;
using Amplitude.AI.Core.Framework;
using Amplitude.AI.Core.Framework.Debug;
using Amplitude.AI.Core.Services;
using Amplitude.AI.Core.Snapshots;
using Amplitude.AI.Core.Snapshots.EntitySnapshots.Empire;
using Amplitude.AI.Core.Snapshots.EntitySnapshots.Galaxy;
using Amplitude.AI.Core.Snapshots.EntitySnapshots.ShipDesign;

namespace Amplitude.AI.Core.Brain.Generators.Front
{
    public class AssistInvade : ContextBasedGenerator<FrontMissionTask, AttackFront, FrontMissionTask_AssistInvade, GenerationData<FrontMissionTask, AttackFront, FrontMissionTask_AssistInvade>>
    {
        public override string Description
        {
            get
            {
                return "Send a invader if taskforce dont have enough manpower to take a system.";
            }
        }

        public override int EvaluationPriority
        {
            get
            {
                return 5;
            }
        }

        public override string LocalizedName
        {
            get
            {
                return "Invade System";
            }
        }

        protected override HeuristicFloat ComputeLocalMotivation(AIPlayer aiPlayer, GenerationData<FrontMissionTask, AttackFront, FrontMissionTask_AssistInvade> taskContext)
        {
            return new HeuristicFloat(1f, "");
        }

        protected override void GenerateDecision(AIPlayer aiPlayer, FrontMissionTask context, AttackFront parent)
        {
            MajorEmpireSnapshot majorEmpireSnapshot = aiPlayer.ControlledEmpireSnapshot as MajorEmpireSnapshot;
            AIPlayer_MajorEmpire aiplayer_MajorEmpire = aiPlayer as AIPlayer_MajorEmpire;
            int num = 0;
            for (int i = 0; i < aiPlayer.AnalysisDataHolder.MyEmpireData.AvailableShipDesigns.Count; i++)
            {
                ShipDesignSnapshot shipDesignSnapshot = aiPlayer.AnalysisDataHolder.MyEmpireData.AvailableShipDesigns[i];
                if (shipDesignSnapshot.ShipRole == ShipPattern.ShipRole.Invader && shipDesignSnapshot.CommandPoints > (float)num)
                {
                    for (int j = 0; j < majorEmpireSnapshot.AvailableHulls.Count; j++)
                    {
                        if (majorEmpireSnapshot.AvailableHulls[j].Name == shipDesignSnapshot.Hull)
                        {
                            num = (int)shipDesignSnapshot.CommandPoints;
                            break;
                        }
                    }
                }
            }
            if (num == 0)
            {
                return;
            }
            FrontMissionTask_AssistInvade frontMissionTask_AssistInvade = aiPlayer.Blackboard.FindFirstRunning<FrontMissionTask_AssistInvade>(context.FrontGuid, context.TargetNode);
            if (frontMissionTask_AssistInvade == null)
            {
                frontMissionTask_AssistInvade = new FrontMissionTask_AssistInvade(aiPlayer.Blackboard.GenerateGUID(), majorEmpireSnapshot.GameEntityGuid, context.FrontGuid);
                frontMissionTask_AssistInvade.TargetNode = context.TargetNode;
                frontMissionTask_AssistInvade.InstanceAllocations = new ResourceAllocation[1];
                frontMissionTask_AssistInvade.InstanceAllocations[0] = new InstanceAllocation_Fleet(aiplayer_MajorEmpire.StrategicDataHolder.MilitaryData.InvaderPattern);
                aiPlayer.Blackboard.Register<FrontMissionTask_AssistInvade>(frontMissionTask_AssistInvade);
            }
            base.ContinueDecision(aiPlayer, base.GetGenerationData(context, parent, frontMissionTask_AssistInvade));
        }

        protected override void GetContexts(AIPlayer aiPlayer)
        {
            this.tasks.Clear();
            aiPlayer.Blackboard.FindAllFrontMissionTasks<FrontMissionTask>(ref this.tasks, BlackboardMessage_Evaluable.EvaluationStatus.Ongoing, true, true);
            for (int i = 0; i < this.tasks.Count; i++)
            {
                base.ContinueDecision(aiPlayer, this.tasks[i]);
            }
        }

        protected override void GetParents(AIPlayer aiPlayer, FrontMissionTask context)
        {
            ISnapshotRepositoryService service = Amplitude.AI.Core.Framework.Services.GetService<ISnapshotRepositoryService>();
            StarSystemNodeSnapshot starSystemNodeSnapshot = null;
            if (!service.GalaxySnapshot.TryGetGameNode<StarSystemNodeSnapshot>(context.TargetNode, out starSystemNodeSnapshot))
            {
                return;
            }
            base.ContinueDecision(aiPlayer, context, aiPlayer.Blackboard.FindFirstRunningFront<AttackFront>(context.FrontGuid));
        }

        public AssistInvade()
        {
        }

        private List<FrontMissionTask> tasks = new List<FrontMissionTask>();
    }
}
